namespace ActiveAllocator.API.Controllers;

using AutoMapper;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using ActiveAllocator.API.Models.Accounts;
using ActiveAllocator.API.Services;
using ActiveAllocator.API.Config;
using System.Collections.Generic;
using ActiveAllocator.API.Entities;
using Microsoft.AspNetCore.Authorization;
using ActiveAllocator.API.Models.Integrations;
using System.Threading.Tasks;

[Authorize]
[ApiController]
[Route("[controller]")]
public class IntegrationsController : ControllerBase
{
    private IIntegrationService _integrationService;
    private IMapper _mapper;
    private readonly AppSettings _appSettings;

    public IntegrationsController(
        IIntegrationService integrationService,
        IMapper mapper,
        IOptions<AppSettings> appSettings)
    {
        _integrationService = integrationService;
        _mapper = mapper;
        _appSettings = appSettings.Value;
    }

    [HttpGet]
    public IActionResult GetAll()
    {
        List<IntegrationDTO> DTOs = new();

        foreach (var item in _integrationService.GetAll())
            DTOs.Add(_mapper.Map<Integration, IntegrationDTO>(item));

        return Ok(DTOs);
    }

    [HttpGet("{id}")]
    public IActionResult GetById(int id)
    {
        return Ok(_mapper.Map<Integration, IntegrationDTO>(_integrationService.GetById(id)));
    }

    [HttpGet("HealthCheck/{id}")]
    public async Task<IActionResult> HealthCheckAsync(int id)
    {
        string version = await _integrationService.IsIntegrationHealthyAsync(id);
        return Ok(new { isHealthy = !string.IsNullOrWhiteSpace(version), version });
    }

    //Task<List<IntegrationRemoteAccountDTO>> GetRemoteAccountsAsync(int integrationAccountId);
    
    [HttpPost]
    public IActionResult Create([FromBody]IntegrationCreateRequest model)
    {
        _integrationService.Create(model);
        return Ok(new { message = "integration created" });
    }

    [HttpPut("{id}")]
    public IActionResult Update(int id, [FromBody]IntegrationUpdateRequest model)
    {
        _integrationService.Update(id, model);
        return Ok(new { message = "integration updated" });
    }

    [HttpDelete("{id}")]
    public IActionResult Delete(int id)
    {
        _integrationService.Delete(id);
        return Ok(new { message = "integration deleted" });
    }

    [HttpGet("Accounts")]
    public IActionResult GetAllAccounts(int? integration_id = null)
    {
        List<IntegrationAccountDTO> DTOs = new();

        foreach (var item in _integrationService.GetAllAccounts(integration_id))
            DTOs.Add(_mapper.Map<IntegrationAccount, IntegrationAccountDTO>(item));

        return Ok(DTOs);
    }

    [HttpGet("Accounts/{id}")]
    public IActionResult GetAccountById(int id)
    {
        return Ok(_mapper.Map<IntegrationAccount, IntegrationAccountDTO>(_integrationService.GetAccountById(id)));
    }

    [HttpPost("Accounts")]
    public async Task<IActionResult> CreateAccount([FromBody]IntegrationAccountCreateRequest model)
    {
        await _integrationService.CreateAccountAsync(model);
        return Ok(new { message = "integration account created" });
    }

    [HttpPut("Accounts/{id}")]
    public async Task<IActionResult> UpdateAccountAsync(int id, [FromBody]IntegrationAccountUpdateRequest model)
    {
        await _integrationService.UpdateAccountAsync(id, model);
        return Ok(new { message = "integration account updated" });
    }

    [HttpDelete("Accounts/{id}")]
    public async Task<IActionResult> DeleteAccount(int id)
    {
        await _integrationService.DeleteAccountAsync(id);
        return Ok(new { message = "integration account deleted" });
    }

    [HttpGet("Accounts/{id}/RemoteAccounts")]
    public async Task<IActionResult> GetRemoteAccountsAsync(int id)
    {
        return Ok(await _integrationService.GetRemoteAccountsAsync(id));
    }

    [HttpPost("Accounts/{id}/RemoteAccounts")]
    public async Task<IActionResult> CreateRemoteAccountAsync(int id, [FromBody]IntegrationRemoteAccountAddRequest model)
    {
        return Ok(await _integrationService.AddRemoteAccountToIntegrationAccountAsync(id, model));
    }

    [HttpDelete("Accounts/{id}/RemoteAccounts/{guid}")]
    public async Task<IActionResult> DeleteRemoteAccountAsync(int id, string guid)
    {
        return Ok(_integrationService.RemoveRemoteAccountFromIntegrationAccountAsync(id, guid));
    }
}