namespace ActiveAllocator.API.Helpers;

using Microsoft.EntityFrameworkCore;
using ActiveAllocator.API.Entities;
using System.Diagnostics;
using System;
using System.Collections.Generic;
using Internal;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Diagnostics.CodeAnalysis;
using Newtonsoft.Json;
using NuGet.DependencyResolver;
using Microsoft.EntityFrameworkCore.Metadata.Builders;
using ActiveAllocator.API.Handlers;

public class APIRequester
{
    private readonly string _baseUrl;
    private readonly uint _timeoutSeconds;
    private readonly CancellationToken _cancellationToken;
    private const string API_KEY_HEADER = "x-api-key";

    public APIRequester(
        string baseUrl,
        uint timeoutSeconds,
        CancellationToken cancellationToken
    )
    {
        _baseUrl = baseUrl;
        _timeoutSeconds = timeoutSeconds;
        _cancellationToken = cancellationToken;
    }

    public async Task<T> GetAsync<T>(string resourcePath, string? apiKey = null)
    {
        return await GetAsync<T>($"{_baseUrl}/{resourcePath}", _timeoutSeconds, apiKey, _cancellationToken);
    }

    public static async Task<T> GetAsync<T>(string url, uint timeoutSeconds, string? apiKey = null, CancellationToken cancellationToken = default)
    {
        TimeSpan timeout = TimeSpan.FromSeconds(timeoutSeconds);
        T ret = default;
        using (var httpClient = new HttpClient())
        {
            if (!string.IsNullOrWhiteSpace(apiKey))
                httpClient.DefaultRequestHeaders.Add(API_KEY_HEADER, apiKey);

            using (var response = await httpClient.GetAsync(url).WaitAsync(timeout, cancellationToken))
            {
                string apiResponse = await response.Content.ReadAsStringAsync(cancellationToken)
                    .WaitAsync(timeout, cancellationToken);
                ret = JsonConvert.DeserializeObject<T>(apiResponse);
            }
        }

        return ret;
    }

    public async Task<T> PostAsync<T>(string resourcePath, object payload, string? apiKey = null)
    {
        return await PostAsync<T>($"{_baseUrl}/{resourcePath}", payload, _timeoutSeconds, apiKey, _cancellationToken);
    }

    public static async Task<T> PostAsync<T>(string url, object payload, uint timeoutSeconds, string? apiKey = null, CancellationToken cancellationToken = default)
    {
        TimeSpan timeout = TimeSpan.FromSeconds(timeoutSeconds);
        T ret = default;
        using (var httpClient = new HttpClient())
        {
            if (!string.IsNullOrWhiteSpace(apiKey))
                httpClient.DefaultRequestHeaders.Add(API_KEY_HEADER, apiKey);

            JsonContent content = JsonContent.Create(payload);
            using (var response = await httpClient.PostAsync(url, content).WaitAsync(timeout, cancellationToken))
            {
                string apiResponse = await response.Content.ReadAsStringAsync(cancellationToken)
                    .WaitAsync(timeout, cancellationToken);
                ret = JsonConvert.DeserializeObject<T>(apiResponse);
            }
        }

        return ret;
    }

    public async Task<T> PutAsync<T>(string resourcePath, object payload, string? apiKey = null)
    {
        return await PutAsync<T>($"{_baseUrl}/{resourcePath}", payload, _timeoutSeconds, apiKey, _cancellationToken);
    }

    public static async Task<T> PutAsync<T>(string url, object payload, uint timeoutSeconds, string? apiKey = null, CancellationToken cancellationToken = default)
    {
        TimeSpan timeout = TimeSpan.FromSeconds(timeoutSeconds);
        T ret = default;
        using (var httpClient = new HttpClient())
        {
            if (!string.IsNullOrWhiteSpace(apiKey))
                httpClient.DefaultRequestHeaders.Add(API_KEY_HEADER, apiKey);

            JsonContent content = JsonContent.Create(payload);
            using (var response = await httpClient.PutAsync(url, content).WaitAsync(timeout, cancellationToken))
            {
                string apiResponse = await response.Content.ReadAsStringAsync(cancellationToken)
                    .WaitAsync(timeout, cancellationToken);
                ret = JsonConvert.DeserializeObject<T>(apiResponse);
            }
        }

        return ret;
    }

    public async Task<T> DeleteAsync<T>(string resourcePath, string? apiKey = null)
    {
        return await DeleteAsync<T>($"{_baseUrl}/{resourcePath}", _timeoutSeconds, apiKey, _cancellationToken);
    }

    public static async Task<T> DeleteAsync<T>(string url, uint timeoutSeconds, string? apiKey = null, CancellationToken cancellationToken = default)
    {
        TimeSpan timeout = TimeSpan.FromSeconds(timeoutSeconds);
        T ret = default;
        using (var httpClient = new HttpClient())
        {
            if (!string.IsNullOrWhiteSpace(apiKey))
                httpClient.DefaultRequestHeaders.Add(API_KEY_HEADER, apiKey);

            using (var response = await httpClient.DeleteAsync(url).WaitAsync(timeout, cancellationToken))
            {
                string apiResponse = await response.Content.ReadAsStringAsync(cancellationToken)
                    .WaitAsync(timeout, cancellationToken);
                ret = JsonConvert.DeserializeObject<T>(apiResponse);
            }
        }

        return ret;
    }
}