namespace ActiveAllocator.API.Services;

using ActiveAllocator.API.Entities;
using ActiveAllocator.API.Config;
using ActiveAllocator.API.Models.Integrations;
using System.Collections.Generic;
using System.Linq;
using System;
using Microsoft.EntityFrameworkCore;
using System.Threading.Tasks;
using Newtonsoft.Json;
using ActiveAllocator.API.Helpers;

public interface IIntegrationService
{
    IEnumerable<Integration> GetAll();
    IEnumerable<IntegrationAccount> GetAllAccounts(int? integrationId = null);
    Integration GetById(int integrationId);
    IntegrationAccount GetAccountById(int accountId);
    void Create(IntegrationCreateRequest model);
    Task CreateAccountAsync(IntegrationAccountCreateRequest model);
    void Update(int integrationId, IntegrationUpdateRequest model);
    Task UpdateAccountAsync(int integrationAccountId, IntegrationAccountUpdateRequest model);
    void Delete(int integrationId);
    Task DeleteAccountAsync(int integrationAccountId);
    Task<string> IsIntegrationHealthyAsync(int integrationId);
    Task<IntegrationAccount> AddRemoteAccountToIntegrationAccountAsync(int integrationAccountId, IntegrationRemoteAccountAddRequest model);
    IntegrationAccount RemoveRemoteAccountFromIntegrationAccountAsync(int integrationAccountId, string remoteAccountGuid);
    Task<List<IntegrationRemoteAccountDTO>> GetRemoteAccountsAsync(int integrationAccountId);
    Task RunTransactionImport(int integrationAccountId);
}

public class IntegrationService : IIntegrationService
{
    private DataContext _context;

    public IntegrationService(
        DataContext context)
    {
        _context = context;
    }

    public IEnumerable<Integration> GetAll()
    {
        return _context.Integrations;
    }

    public IEnumerable<IntegrationAccount> GetAllAccounts(int? integrationId = null)
    {
        if (integrationId.HasValue)
            return _context.IntegrationAccounts
                .Include(a => a.Integration)
                .Include(a => a.AccountMappings)
                    .ThenInclude(a => a.LinkedAccount)
                .Where(a => a.Integration.Id == integrationId.Value);
        else
            return _context.IntegrationAccounts
                .Include(a => a.Integration)
                .Include(a => a.AccountMappings)
                    .ThenInclude(a => a.LinkedAccount);
    }

    public Integration GetById(int integrationId)
    {
        return GetIntegration(integrationId);
    }

    public IntegrationAccount GetAccountById(int accountId)
    {
        return GetIntegrationAccount(accountId);
    }

    public async void Create(IntegrationCreateRequest model)
    {
        if (_context.Integrations.Any(a => a.Name.ToUpper() == model.Name.ToUpper()))
            throw new AppException("Integration with name '" + model.Name + "' already exists");

        if (_context.Integrations.Any(a => a.ApiUrl.ToUpper() == model.ApiUrl.ToUpper()))
            throw new AppException("Integration with API Url '" + model.ApiUrl + "' already exists");

        Integration integration = new Integration {
            Name = model.Name,
            ApiUrl = model.ApiUrl,
            CreatedOn = DateTime.UtcNow,
            Type = model.Type,
            Version = ""
        };

        _context.Integrations.Add(integration);
        _context.SaveChanges();

        IsIntegrationHealthyAsync(integration.Id);
    }

    public async Task CreateAccountAsync(IntegrationAccountCreateRequest model)
    {
        // Validate model info
        if (_context.IntegrationAccounts
            .Include(a => a.Integration)
            .Any(a => a.Name.ToUpper() == model.Name.ToUpper() && a.Integration.Id == model.IntegrationId))
            throw new AppException($"IntegrationAccount with name '{model.Name}' already exists on integration '{model.IntegrationId}'");

        Integration integration = _context.Integrations.Find(model.IntegrationId) ?? throw new AppException($"Integration with id '{model.IntegrationId}' not found");

        /*Account account = _context.Accounts.Find(model.LinkedAccountId) ?? throw new AppException($"Account with id '{model.LinkedAccountId}' not found");*/

        // Get API Key from integration
        string apiKey = await RegisterRemoteIntegrationAccount(integration.Id, new IntegrationAccountCredentialsDTO() {
            Username = model.Username,
            Password = model.Password,
            MFAKey = model.MFAKey
        });

        // Create entity and insert into database
        IntegrationAccount integrationAccount = new IntegrationAccount {
            Name = model.Name,
            ApiKey = apiKey,
            CreatedOn = DateTime.UtcNow,
            Integration = integration,
            AccountMappings = new(),
            Enabled = model.Enabled,
            RunInterval = model.RunInterval,
            LastRun = DateTime.UnixEpoch.ToUniversalTime()
        };

        _context.IntegrationAccounts.Add(integrationAccount);
        _context.SaveChanges();
    }

    public void Update(int integrationId, IntegrationUpdateRequest model)
    {
        Integration integration = GetIntegration(integrationId);

        // validate
        if (model.Name != integration.Name && _context.Integrations.Any(x => x.Name == model.Name))
            throw new AppException("Integration with the name '" + model.Name + "' already exists");

        // validate
        if (model.ApiUrl != integration.ApiUrl && _context.Integrations.Any(x => x.ApiUrl == model.ApiUrl))
            throw new AppException("Integration with the API Url '" + model.ApiUrl + "' already exists");

        // Name
        if (!string.IsNullOrWhiteSpace(model.Name))
            integration.Name = model.Name;

        // Api Url
        if (!string.IsNullOrWhiteSpace(model.ApiUrl))
            integration.ApiUrl = model.ApiUrl;

        // Types
        if (model.Type.HasValue)
        {
            integration.Type = model.Type.Value;
        }

        _context.Integrations.Update(integration);
        _context.SaveChanges();
    }

    public async Task UpdateAccountAsync(int integrationAccountId, IntegrationAccountUpdateRequest model)
    {
        IntegrationAccount integrationAccount = GetIntegrationAccount(integrationAccountId);

        // Check if new name is valid
        if (model.Name != integrationAccount.Name && _context.IntegrationAccounts
            .Include(a => a.Integration)
            .Any(a => a.Name.ToUpper() == model.Name.ToUpper() && a.Integration.Id == integrationAccount.Integration.Id))
            throw new AppException($"IntegrationAccount with name '{model.Name}' already exists on integration '{integrationAccount.Integration.Id}'");

        /*if (model.LinkedAccountId.HasValue)
        {
            Account account = _context.Accounts.Find(model.LinkedAccountId.Value) ?? throw new AppException($"Could not find the account specified by LinkedAccountId");
            integrationAccount.LinkedAccount = account;
        }*/

        if (model.Enabled.HasValue)
            integrationAccount.Enabled = model.Enabled.Value;

        if (model.RunInterval.HasValue)
            integrationAccount.RunInterval = model.RunInterval.Value;

        // Check if username, password, and MFAKey are set
        if (model.Username != null || model.Password != null || model.MFAKey != null)
        {
            if (model.Username == null || model.Password == null || model.MFAKey == null)
                throw new AppException($"IntegrationAccount credential change requires all new credentials");

            string apiKey = await UpdateRemoteIntegrationAccount(integrationAccount.Id, new IntegrationAccountCredentialsDTO() {
                Username = model.Username,
                Password = model.Password,
                MFAKey = model.MFAKey
            });

            integrationAccount.ApiKey = apiKey;
        }

        _context.IntegrationAccounts.Update(integrationAccount);
        _context.SaveChanges();
    }

    public void Delete(int integrationId)
    {
        var integration = GetIntegration(integrationId);
        _context.Integrations.Remove(integration);
        _context.SaveChanges();
    }

    public async Task DeleteAccountAsync(int integrationAccountId)
    {
        var integrationAccount = _context.IntegrationAccounts.Find(integrationAccountId) ?? throw new AppException($"Could not find the integration account specified to delete");
        await DeleteRemoteIntegrationAccount(integrationAccount.Id);
        _context.IntegrationAccounts.Remove(integrationAccount);
        _context.SaveChanges();
    }

    public async Task<string> IsIntegrationHealthyAsync(int integrationId)
    {
        Integration integration = GetIntegration(integrationId);

        IntegrationApplicationVersionDTO dto = await APIRequester.GetAsync<IntegrationApplicationVersionDTO>($"{integration.ApiUrl}/AppInfo/ApplicationVersion", 3);

        integration.Version = dto.Version;
        _context.Integrations.Update(integration);
        _context.SaveChanges();

        return dto.Version;
    }

    public async Task<IntegrationAccount> AddRemoteAccountToIntegrationAccountAsync(int integrationAccountId, IntegrationRemoteAccountAddRequest model)
    {
        IntegrationAccount integrationAccount = GetIntegrationAccount(integrationAccountId);

        Account account = _context.Accounts.Find(model.LinkedAccountId) ?? throw new AppException($"Linked Account {model.LinkedAccountId} was not found");

        if (integrationAccount.AccountMappings.Any(a => a.ExternalId == model.Id))
            throw new AppException($"Remote Account with GUID {model.Id} is already linked");

        IntegrationAccountMapping mapping = new()
        {
            ExternalId = model.Id,
            Name = model.Name,
            Enabled = model.Enabled,
            LinkedAccount = account,
        };

        IntegrationRemoteAccountDTO dto = await APIRequester.GetAsync<IntegrationRemoteAccountDTO>($"{integrationAccount.Integration.ApiUrl}/Accounts/{model.Id}", 3, integrationAccount.ApiKey);

        if (dto != null)
        {
            mapping.Balance = dto.Balance;
            mapping.AvailableBalance = dto.AvailableBalance;
            mapping.AccountType = dto.AccountType;
        }

        integrationAccount.AccountMappings.Add(mapping);
        _context.IntegrationAccounts.Update(integrationAccount);
        _context.SaveChanges();
        
        return integrationAccount;
    }

    public IntegrationAccount RemoveRemoteAccountFromIntegrationAccountAsync(int integrationAccountId, string remoteAccountGuid)
    {
        IntegrationAccount integrationAccount = GetIntegrationAccount(integrationAccountId);

        IntegrationAccountMapping mapping = integrationAccount.AccountMappings.First(a => a.ExternalId == remoteAccountGuid) ?? throw new AppException($"Remote Account {remoteAccountGuid} was not mapped to integration account {integrationAccountId}");

        integrationAccount.AccountMappings.Remove(mapping);

        _context.IntegrationAccounts.Update(integrationAccount);
        _context.SaveChanges();
        
        return integrationAccount;
    }

    public async Task<List<IntegrationRemoteAccountDTO>> GetRemoteAccountsAsync(int integrationAccountId)
    {
        IntegrationAccount integrationAccount = GetIntegrationAccount(integrationAccountId);
        return await APIRequester.GetAsync<List<IntegrationRemoteAccountDTO>>($"{integrationAccount.Integration.ApiUrl}/Accounts", 3, integrationAccount.ApiKey);
    }

    public async Task RunTransactionImport(int integrationAccountId)
    {
        IntegrationAccount integrationAccount = GetAccountById(integrationAccountId);
        Integration integration = integrationAccount.Integration;

        //Pull Transactions
    }

    // helper methods

    private async Task<string> UpdateRemoteIntegrationAccount(int integrationAccountId, IntegrationAccountCredentialsDTO credentialsDTO)
    {
        IntegrationAccount integrationAccount = GetIntegrationAccount(integrationAccountId);
        return await APIRequester.PutAsync<string>($"{integrationAccount.Integration.ApiUrl}/Users", credentialsDTO, 3, integrationAccount.ApiKey);
    }

    private async Task DeleteRemoteIntegrationAccount(int integrationAccountId)
    {
        IntegrationAccount integrationAccount = GetIntegrationAccount(integrationAccountId);
        await APIRequester.DeleteAsync<object>($"{integrationAccount.Integration.ApiUrl}/Users", 3, integrationAccount.ApiKey);
    }

    private async Task<string> RegisterRemoteIntegrationAccount(int integrationId, IntegrationAccountCredentialsDTO credentialsDTO)
    {
        Integration integration = GetIntegration(integrationId);
        Console.WriteLine("RegisterRemoteIntegrationAccount: Got integration about to post");
        //string url = "https://vv.requestcatcher.com/test";
        string url = $"{integration.ApiUrl}/Users/register";
        IntegrationAccountCreateResponse response = await APIRequester.PostAsync<IntegrationAccountCreateResponse>(url, credentialsDTO, 3);
        return response.ApiKey;
    }

    private Integration GetIntegration(int id)
    {
        var integration = _context.Integrations.Find(id) ?? throw new KeyNotFoundException("Integration not found");
        return integration;
    }

    private IntegrationAccount GetIntegrationAccount(int id)
    {
        return _context.IntegrationAccounts
            .Include(a => a.Integration)
            .Include(a => a.AccountMappings)
                .ThenInclude(a => a.LinkedAccount)
            .FirstOrDefault(a => a.Id == id)
            ?? throw new KeyNotFoundException("IntegrationAccount not found");
    }
}